@@ -1,6 +1,7 @@
 /* poppler-annot.cc: glib interface to poppler
  *
  * Copyright (C) 2007 Inigo Martinez <inigomartinez@gmail.com>
+ * Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +21,13 @@
 #include "poppler.h"
 #include "poppler-private.h"
 
-typedef struct _PopplerAnnotClass         PopplerAnnotClass;
-typedef struct _PopplerAnnotMarkupClass   PopplerAnnotMarkupClass;
-typedef struct _PopplerAnnotFreeTextClass PopplerAnnotFreeTextClass;
-typedef struct _PopplerAnnotTextClass     PopplerAnnotTextClass;
+typedef struct _PopplerAnnotClass               PopplerAnnotClass;
+typedef struct _PopplerAnnotMarkupClass         PopplerAnnotMarkupClass;
+typedef struct _PopplerAnnotFreeTextClass       PopplerAnnotFreeTextClass;
+typedef struct _PopplerAnnotTextClass           PopplerAnnotTextClass;
+typedef struct _PopplerAnnotFileAttachmentClass PopplerAnnotFileAttachmentClass;
+typedef struct _PopplerAnnotMovieClass          PopplerAnnotMovieClass;
+typedef struct _PopplerAnnotScreenClass         PopplerAnnotScreenClass;
 
 struct _PopplerAnnot
 {
@@ -66,10 +70,48 @@ struct _PopplerAnnotFreeTextClass
   PopplerAnnotMarkupClass parent_class;
 };
 
+struct _PopplerAnnotFileAttachment
+{
+  PopplerAnnotMarkup parent_instance;
+};
+
+struct _PopplerAnnotFileAttachmentClass
+{
+  PopplerAnnotMarkupClass parent_class;
+};
+
+struct _PopplerAnnotMovie
+{
+  PopplerAnnot  parent_instance;
+
+  PopplerMovie *movie;
+};
+
+struct _PopplerAnnotMovieClass
+{
+  PopplerAnnotClass parent_class;
+};
+
+struct _PopplerAnnotScreen
+{
+  PopplerAnnot  parent_instance;
+
+  PopplerAction *action;
+};
+
+struct _PopplerAnnotScreenClass
+{
+  PopplerAnnotClass parent_class;
+};
+
+
 G_DEFINE_TYPE (PopplerAnnot, poppler_annot, G_TYPE_OBJECT)
 G_DEFINE_TYPE (PopplerAnnotMarkup, poppler_annot_markup, POPPLER_TYPE_ANNOT)
 G_DEFINE_TYPE (PopplerAnnotText, poppler_annot_text, POPPLER_TYPE_ANNOT_MARKUP)
 G_DEFINE_TYPE (PopplerAnnotFreeText, poppler_annot_free_text, POPPLER_TYPE_ANNOT_MARKUP)
+G_DEFINE_TYPE (PopplerAnnotFileAttachment, poppler_annot_file_attachment, POPPLER_TYPE_ANNOT_MARKUP)
+G_DEFINE_TYPE (PopplerAnnotMovie, poppler_annot_movie, POPPLER_TYPE_ANNOT)
+G_DEFINE_TYPE (PopplerAnnotScreen, poppler_annot_screen, POPPLER_TYPE_ANNOT)
 
 static void
 poppler_annot_finalize (GObject *object)
@@ -157,6 +199,114 @@ _poppler_annot_free_text_new (Annot *annot)
   return poppler_annot;
 }
 
+static void
+poppler_annot_file_attachment_init (PopplerAnnotFileAttachment *poppler_annot)
+{
+}
+
+static void
+poppler_annot_file_attachment_class_init (PopplerAnnotFileAttachmentClass *klass)
+{
+}
+
+PopplerAnnot *
+_poppler_annot_file_attachment_new (Annot *annot)
+{
+  PopplerAnnot *poppler_annot;
+
+  poppler_annot = POPPLER_ANNOT (g_object_new (POPPLER_TYPE_ANNOT_FILE_ATTACHMENT, NULL));
+  poppler_annot->annot = annot;
+
+  return poppler_annot;
+}
+
+
+static void
+poppler_annot_movie_finalize (GObject *object)
+{
+  PopplerAnnotMovie *annot_movie = POPPLER_ANNOT_MOVIE (object);
+
+  if (annot_movie->movie) {
+    g_object_unref (annot_movie->movie);
+    annot_movie->movie = NULL;
+  }
+
+  G_OBJECT_CLASS (poppler_annot_movie_parent_class)->finalize (object);
+}
+
+static void
+poppler_annot_movie_init (PopplerAnnotMovie *poppler_annot)
+{
+}
+
+static void
+poppler_annot_movie_class_init (PopplerAnnotMovieClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+
+  gobject_class->finalize = poppler_annot_movie_finalize;
+}
+
+PopplerAnnot *
+_poppler_annot_movie_new (Annot *annot)
+{
+  PopplerAnnot *poppler_annot;
+  AnnotMovie   *annot_movie;
+
+  poppler_annot = POPPLER_ANNOT (g_object_new (POPPLER_TYPE_ANNOT_MOVIE, NULL));
+  poppler_annot->annot = annot;
+
+  annot_movie = static_cast<AnnotMovie *>(poppler_annot->annot);
+  POPPLER_ANNOT_MOVIE (poppler_annot)->movie = _poppler_movie_new (annot_movie->getMovie());
+
+  return poppler_annot;
+}
+
+static void
+poppler_annot_screen_finalize (GObject *object)
+{
+  PopplerAnnotScreen *annot_screen = POPPLER_ANNOT_SCREEN (object);
+
+  if (annot_screen->action) {
+    poppler_action_free (annot_screen->action);
+    annot_screen->action = NULL;
+  }
+
+  G_OBJECT_CLASS (poppler_annot_screen_parent_class)->finalize (object);
+}
+
+static void
+poppler_annot_screen_init (PopplerAnnotScreen *poppler_annot)
+{
+}
+
+static void
+poppler_annot_screen_class_init (PopplerAnnotScreenClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+
+  gobject_class->finalize = poppler_annot_screen_finalize;
+}
+
+PopplerAnnot *
+_poppler_annot_screen_new (Annot *annot)
+{
+  PopplerAnnot *poppler_annot;
+  AnnotScreen  *annot_screen;
+  LinkAction   *action;
+
+  poppler_annot = POPPLER_ANNOT (g_object_new (POPPLER_TYPE_ANNOT_SCREEN, NULL));
+  poppler_annot->annot = annot;
+
+  annot_screen = static_cast<AnnotScreen *>(poppler_annot->annot);
+  action = annot_screen->getAction();
+  if (action)
+    POPPLER_ANNOT_SCREEN (poppler_annot)->action = _poppler_action_new (NULL, action, NULL);
+
+  return poppler_annot;
+}
+
+
 /* Public methods */
 /**
  * poppler_annot_get_annot_type:
@@ -251,6 +401,31 @@ poppler_annot_get_contents (PopplerAnnot *poppler_annot)
   return contents ? _poppler_goo_string_to_utf8 (contents) : NULL;
 }
 
+/**
+ * poppler_annot_set_contents:
+ * @poppler_annot: a #PopplerAnnot
+ * @contents: a text string containing the new contents
+ *
+ * Sets the contents of @poppler_annot to the given value,
+ * replacing the current contents.
+ **/
+void
+poppler_annot_set_contents (PopplerAnnot *poppler_annot,
+			    const gchar  *contents)
+{
+  GooString *goo_tmp;
+  gchar *tmp;
+  gsize length = 0;
+  
+  g_return_if_fail (POPPLER_IS_ANNOT (poppler_annot));
+
+  tmp = contents ? g_convert (contents, -1, "UTF-16BE", "UTF-8", NULL, &length, NULL) : NULL;
+  goo_tmp = new GooString (tmp, length);
+  g_free (tmp);
+  poppler_annot->annot->setContents (goo_tmp);
+  delete (goo_tmp);
+}
+
 /**
  * poppler_annot_get_name:
  * @poppler_annot: a #PopplerAnnot
@@ -276,7 +451,9 @@ poppler_annot_get_name (PopplerAnnot *poppler_annot)
  * poppler_annot_get_modified:
  * @poppler_annot: a #PopplerAnnot
  *
- * Retrieves the last modification data of @poppler_annot.
+ * Retrieves the last modification data of @poppler_annot. The returned
+ * string will be either a PDF format date or a text string.
+ * See also #poppler_date_parse()
  *
  * Return value: a new allocated string with the last modification data of
  *               @poppler_annot. It must be freed with g_free() when done.
@@ -317,41 +494,66 @@ poppler_annot_get_flags (PopplerAnnot *poppler_annot)
  * Retrieves the color of @poppler_annot.
  *
  * Return value: a new allocated #PopplerColor with the color values of
- *               @poppler_annot. It must be freed with g_free() when done.
+ *               @poppler_annot, or %NULL. It must be freed with g_free() when done.
  **/
 PopplerColor *
 poppler_annot_get_color (PopplerAnnot *poppler_annot)
 {
   AnnotColor *color;
+  PopplerColor *poppler_color = NULL;
 
   g_return_val_if_fail (POPPLER_IS_ANNOT (poppler_annot), NULL);
 
-  if ((color = poppler_annot->annot->getColor ())) {
-    PopplerColor *poppler_color;
-    double *values = color->getValues ();
+  color = poppler_annot->annot->getColor ();
+
+  if (color) {
+    const double *values = color->getValues ();
 
     switch (color->getSpace ())
       {
       case AnnotColor::colorGray:
         poppler_color = g_new (PopplerColor, 1);
-        poppler_color->red = (guint16) values[0];
-        poppler_color->green = (guint16) values[0];
-        poppler_color->blue = (guint16) values[0];
-        return poppler_color;
+	
+        poppler_color->red = (guint16) (values[0] * 65535);
+        poppler_color->green = poppler_color->red;
+        poppler_color->blue = poppler_color->red;
+
+	break;
       case AnnotColor::colorRGB:
         poppler_color = g_new (PopplerColor, 1);
-        poppler_color->red = (guint16) values[0];
-        poppler_color->green = (guint16) values[1];
-        poppler_color->blue = (guint16) values[2];
-        return poppler_color;
-      case AnnotColor::colorTransparent:
+	
+        poppler_color->red = (guint16) (values[0] * 65535);
+        poppler_color->green = (guint16) (values[1] * 65535);
+        poppler_color->blue = (guint16) (values[2] * 65535);
+
+	break;
       case AnnotColor::colorCMYK:
-      default:
-        g_warning ("Unsupported Annot Color");
+        g_warning ("Unsupported Annot Color: colorCMYK");
+      case AnnotColor::colorTransparent:
+        break;
       }
   }
 
-  return NULL;
+  return poppler_color;
+}
+
+/**
+ * poppler_annot_get_page_index:
+ * @poppler_annot: a #PopplerAnnot
+ *
+ * Returns the page index to which @poppler_annot is associated, or -1 if unknown
+ *
+ * Return value: page index or -1
+ **/
+gint
+poppler_annot_get_page_index (PopplerAnnot *poppler_annot)
+{
+  gint page_num;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT (poppler_annot), -1);
+
+  page_num = poppler_annot->annot->getPageNum();
+  return page_num <= 0 ? -1 : page_num - 1;
 }
 
 /* PopplerAnnotMarkup */
@@ -378,6 +580,26 @@ poppler_annot_markup_get_label (PopplerAnnotMarkup *poppler_annot)
   return text ? _poppler_goo_string_to_utf8 (text) : NULL;
 }
 
+/**
+ * poppler_annot_markup_has_popup:
+ * @poppler_annot: a #PopplerAnnotMarkup
+ *
+ * Return %TRUE if the markup annotation has a popup window associated
+ *
+ * Return value: %TRUE, if @poppler_annot has popup, %FALSE otherwise
+ **/
+gboolean
+poppler_annot_markup_has_popup (PopplerAnnotMarkup *poppler_annot)
+{
+  AnnotMarkup *annot;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), FALSE);
+
+  annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+
+  return annot->getPopup () != NULL;
+}
+
 /**
  * poppler_annot_markup_get_popup_is_open:
  * @poppler_annot: a #PopplerAnnotMarkup
@@ -403,6 +625,41 @@ poppler_annot_markup_get_popup_is_open (PopplerAnnotMarkup *poppler_annot)
   return FALSE;
 }
 
+/**
+ * poppler_annot_markup_get_popup_rectangle:
+ * @poppler_annot: a #PopplerAnnotMarkup
+ * @poppler_rect: a #PopplerRectangle to store the popup rectangle
+ *
+ * Retrieves the rectangle of the popup annot related to @poppler_annot.
+ *
+ * Return value: %TRUE if #PopplerRectangle was correctly filled,
+ *               %FALSE otherwise
+ **/
+gboolean
+poppler_annot_markup_get_popup_rectangle (PopplerAnnotMarkup *poppler_annot,
+					  PopplerRectangle   *poppler_rect)
+{
+  AnnotMarkup *annot;
+  Annot *annot_popup;
+  PDFRectangle *annot_rect;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), FALSE);
+  g_return_val_if_fail (poppler_rect != NULL, FALSE);
+
+  annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+  annot_popup = annot->getPopup ();
+  if (!annot_popup)
+    return FALSE;
+
+  annot_rect = annot_popup->getRect ();
+  poppler_rect->x1 = annot_rect->x1;
+  poppler_rect->x2 = annot_rect->x2;
+  poppler_rect->y1 = annot_rect->y1;
+  poppler_rect->y2 = annot_rect->y2;
+
+  return TRUE;
+}
+
 /**
 * poppler_annot_markup_get_opacity:
 * @poppler_annot: a #PopplerAnnotMarkup
@@ -428,22 +685,22 @@ poppler_annot_markup_get_date (PopplerAnnotMarkup *poppler_annot)
 {
   AnnotMarkup *annot;
   GooString *annot_date;
+  time_t timet;
 
   g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), NULL);
 
   annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+  annot_date = annot->getDate ();
+  if (!annot_date)
+    return NULL;
 
-  if ((annot_date = annot->getDate ())) {
-    GDateYear year = g_ascii_digit_value (annot_date->getChar(2)) * 1000 +
-                     g_ascii_digit_value (annot_date->getChar(3)) * 100 +
-                     g_ascii_digit_value (annot_date->getChar(4)) * 10 +
-                     g_ascii_digit_value (annot_date->getChar(5));
-    GDateMonth month = (GDateMonth) (g_ascii_digit_value (annot_date->getChar(6)) * 10 +
-                                     g_ascii_digit_value (annot_date->getChar(7)));
-    GDateDay day = g_ascii_digit_value (annot_date->getChar(8)) * 10 +
-                   g_ascii_digit_value (annot_date->getChar(9));
+  if (_poppler_convert_pdf_date_to_gtime (annot_date, &timet)) {
+    GDate *date;
 
-    return g_date_new_dmy (day, month, year);
+    date = g_date_new ();
+    g_date_set_time_t (date, timet);
+
+    return date;
   }
 
   return NULL;
@@ -696,20 +953,61 @@ poppler_annot_free_text_get_callout_line (PopplerAnnotFreeText *poppler_annot)
   return NULL;
 }
 
-/* PopplerAnnotCalloutLine */
-GType
-poppler_annot_callout_line_get_type (void)
+/* PopplerAnnotFileAttachment */
+/**
+ * poppler_annot_file_attachment_get_attachment:
+ * @poppler_annot: a #PopplerAnnotFileAttachment
+ *
+ * Creates a #PopplerAttachment for the file of the file attachment annotation @annot.
+ * The #PopplerAttachment must be unrefed with g_object_unref by the caller.
+ *
+ * Return value: @PopplerAttachment
+ **/
+PopplerAttachment *
+poppler_annot_file_attachment_get_attachment (PopplerAnnotFileAttachment *poppler_annot)
 {
-  static GType our_type = 0;
+  AnnotFileAttachment *annot;
+  PopplerAttachment *attachment;
 
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PopplerAnnotCalloutLine",
-                                             (GBoxedCopyFunc) poppler_annot_callout_line_copy,
-                                             (GBoxedFreeFunc) poppler_annot_callout_line_free);
+  g_return_val_if_fail (POPPLER_IS_ANNOT_FILE_ATTACHMENT (poppler_annot), NULL);
 
-  return our_type;
+  annot = static_cast<AnnotFileAttachment *>(POPPLER_ANNOT (poppler_annot)->annot);
+
+  EmbFile *emb_file = new EmbFile (annot->getFile(), annot->getContents());
+  attachment = _poppler_attachment_new (emb_file);
+  delete emb_file;
+
+  return attachment;
 }
 
+/**
+ * poppler_annot_file_attachment_get_name:
+ * @poppler_annot: a #PopplerAnnotFileAttachment
+ *
+ * Retrieves the name of @poppler_annot.
+ *
+ * Return value: a new allocated string with the name of @poppler_annot. It must
+ *               be freed with g_free() when done.
+ **/
+gchar *
+poppler_annot_file_attachment_get_name (PopplerAnnotFileAttachment *poppler_annot)
+{
+  AnnotFileAttachment *annot;
+  GooString *name;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_FILE_ATTACHMENT (poppler_annot), NULL);
+
+  annot = static_cast<AnnotFileAttachment *>(POPPLER_ANNOT (poppler_annot)->annot);
+  name = annot->getName ();
+
+  return name ? _poppler_goo_string_to_utf8 (name) : NULL;
+}
+
+/* PopplerAnnotCalloutLine */
+POPPLER_DEFINE_BOXED_TYPE (PopplerAnnotCalloutLine, poppler_annot_callout_line,
+			   poppler_annot_callout_line_copy,
+			   poppler_annot_callout_line_free)
+
 /**
  * poppler_annot_callout_line_new:
  *
@@ -757,3 +1055,59 @@ poppler_annot_callout_line_free (PopplerAnnotCalloutLine *callout)
 {
   g_free (callout);
 }
+
+
+/* PopplerAnnotMovie */
+/**
+* poppler_annot_movie_get_title:
+* @poppler_annot: a #PopplerAnnotMovie
+*
+* Retrieves the movie title of @poppler_annot.
+*
+* Return value: the title text of @poppler_annot.
+*/
+gchar *
+poppler_annot_movie_get_title (PopplerAnnotMovie *poppler_annot)
+{
+  AnnotMovie *annot;
+  GooString *title;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_MOVIE (poppler_annot), NULL);
+
+  annot = static_cast<AnnotMovie *>(POPPLER_ANNOT (poppler_annot)->annot);
+
+  title = annot->getTitle ();
+
+  return title ? _poppler_goo_string_to_utf8 (title) : NULL;
+}
+
+/**
+* poppler_annot_movie_get_movie:
+* @poppler_annot: a #PopplerAnnotMovie
+*
+* Retrieves the movie object (PopplerMovie) stored in the @poppler_annot.
+*
+* Return value: the movie object stored in the @poppler_annot. The returned
+*               object is owned by #PopplerAnnotMovie and should not be freed
+*/
+PopplerMovie *
+poppler_annot_movie_get_movie (PopplerAnnotMovie *poppler_annot)
+{
+  return poppler_annot->movie;
+}
+
+/* PopplerAnnotScreen */
+/**
+ * poppler_annot_screen_get_action:
+ * @poppler_annot: a #PopplerAnnotScreen
+ *
+ * Retrieves the action (#PopplerAction) that shall be performed when @poppler_annot is activated
+ *
+ * Return value: the action to perform. The returned
+ *               object is owned by @poppler_annot and should not be freed
+ */
+PopplerAction *
+poppler_annot_screen_get_action (PopplerAnnotScreen *poppler_annot)
+{
+  return poppler_annot->action;
+}
