@@ -13,14 +13,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __POPPLER_GLIB_H__
 #define __POPPLER_GLIB_H__
 
 #include <glib-object.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 
 G_BEGIN_DECLS
 
@@ -31,7 +30,10 @@ GQuark poppler_error_quark (void);
 typedef enum
 {
   POPPLER_ERROR_INVALID,
-  POPPLER_ERROR_ENCRYPTED
+  POPPLER_ERROR_ENCRYPTED,
+  POPPLER_ERROR_OPEN_FILE,
+  POPPLER_ERROR_BAD_CATALOG,
+  POPPLER_ERROR_DAMAGED
 } PopplerError;
 
 typedef enum
@@ -77,21 +79,36 @@ typedef enum
   POPPLER_SELECTION_LINE
 } PopplerSelectionStyle;
 
-typedef struct _PopplerDocument         PopplerDocument;
-typedef struct _PopplerIndexIter        PopplerIndexIter;
-typedef struct _PopplerFontsIter        PopplerFontsIter;
-typedef struct _PopplerRectangle        PopplerRectangle;
-typedef struct _PopplerLinkMapping      PopplerLinkMapping;
-typedef struct _PopplerPageTransition   PopplerPageTransition;
-typedef struct _PopplerImageMapping     PopplerImageMapping;
-typedef struct _PopplerFormFieldMapping PopplerFormFieldMapping;
-typedef struct _PopplerPage             PopplerPage;
-typedef struct _PopplerFontInfo         PopplerFontInfo;
-typedef struct _PopplerPSFile           PopplerPSFile;
-typedef union  _PopplerAction           PopplerAction;
-typedef struct _PopplerDest             PopplerDest;
-typedef struct _PopplerFormField        PopplerFormField;
-typedef struct _PopplerAttachment       PopplerAttachment;
+typedef struct _PopplerDocument            PopplerDocument;
+typedef struct _PopplerIndexIter           PopplerIndexIter;
+typedef struct _PopplerFontsIter           PopplerFontsIter;
+typedef struct _PopplerLayersIter          PopplerLayersIter;
+typedef struct _PopplerRectangle           PopplerRectangle;
+typedef struct _PopplerColor               PopplerColor;
+typedef struct _PopplerLinkMapping         PopplerLinkMapping;
+typedef struct _PopplerPageTransition      PopplerPageTransition;
+typedef struct _PopplerImageMapping        PopplerImageMapping;
+typedef struct _PopplerFormFieldMapping    PopplerFormFieldMapping;
+typedef struct _PopplerAnnotMapping        PopplerAnnotMapping;
+typedef struct _PopplerPage                PopplerPage;
+typedef struct _PopplerFontInfo            PopplerFontInfo;
+typedef struct _PopplerLayer               PopplerLayer;
+typedef struct _PopplerPSFile              PopplerPSFile;
+typedef union  _PopplerAction              PopplerAction;
+typedef struct _PopplerDest                PopplerDest;
+typedef struct _PopplerActionLayer         PopplerActionLayer;
+typedef struct _PopplerFormField           PopplerFormField;
+typedef struct _PopplerAttachment          PopplerAttachment;
+typedef struct _PopplerMovie               PopplerMovie;
+typedef struct _PopplerMedia               PopplerMedia;
+typedef struct _PopplerAnnot               PopplerAnnot;
+typedef struct _PopplerAnnotMarkup         PopplerAnnotMarkup;
+typedef struct _PopplerAnnotText           PopplerAnnotText;
+typedef struct _PopplerAnnotFreeText       PopplerAnnotFreeText;
+typedef struct _PopplerAnnotFileAttachment PopplerAnnotFileAttachment;
+typedef struct _PopplerAnnotMovie          PopplerAnnotMovie;
+typedef struct _PopplerAnnotScreen         PopplerAnnotScreen;
+typedef struct _PopplerAnnotCalloutLine    PopplerAnnotCalloutLine;
 
 typedef enum
 {
@@ -108,9 +125,14 @@ G_END_DECLS
 #include "poppler-features.h"
 #include "poppler-document.h"
 #include "poppler-page.h"
+#include "poppler-layer.h"
 #include "poppler-action.h"
 #include "poppler-form-field.h"
 #include "poppler-enums.h"
 #include "poppler-attachment.h"
+#include "poppler-annot.h"
+#include "poppler-date.h"
+#include "poppler-movie.h"
+#include "poppler-media.h"
 
 #endif /* __POPPLER_GLIB_H__ */
