@@ -6,6 +6,20 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2008, 2010 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
@@ -24,7 +38,7 @@
 // Object
 //------------------------------------------------------------------------
 
-char *objTypeNames[numObjTypes] = {
+static char *objTypeNames[numObjTypes] = {
   "boolean",
   "integer",
   "real",
@@ -209,6 +223,9 @@ void Object::print(FILE *f) {
   case objNone:
     fprintf(f, "<none>");
     break;
+  case objUint:
+    fprintf(f, "%u", uintg);
+    break;
   }
 }
 
