@@ -11,9 +11,12 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2006, 2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2009, 2010 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007 Ilmari Heikkinen <ilmari.heikkinen@gmail.com>
 // Copyright (C) 2009 Shen Liang <shenzhuxi@gmail.com>
+// Copyright (C) 2009 Stefan Thomas <thomas@eload24.com>
+// Copyright (C) 2010 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2010 Harry Roberts <harry.roberts@midnight-labs.org>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -28,11 +31,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
 #include "goo/gmem.h"
 #include "SplashErrorCodes.h"
 #include "SplashBitmap.h"
 #include "poppler/Error.h"
-#include "PNGWriter.h"
+#include "goo/JpegWriter.h"
+#include "goo/PNGWriter.h"
+#include "goo/ImgWriter.h"
 
 //------------------------------------------------------------------------
 // SplashBitmap
@@ -46,26 +52,48 @@ SplashBitmap::SplashBitmap(int widthA, int heightA, int rowPad,
   mode = modeA;
   switch (mode) {
   case splashModeMono1:
-    rowSize = (width + 7) >> 3;
+    if (width > 0) {
+      rowSize = (width + 7) >> 3;
+    } else {
+      rowSize = -1;
+    }
     break;
   case splashModeMono8:
-    rowSize = width;
+    if (width > 0) {
+      rowSize = width;
+    } else {
+      rowSize = -1;
+    }
     break;
   case splashModeRGB8:
   case splashModeBGR8:
-    rowSize = width * 3;
+    if (width > 0 && width <= INT_MAX / 3) {
+      rowSize = width * 3;
+    } else {
+      rowSize = -1;
+    }
     break;
   case splashModeXBGR8:
-    rowSize = width * 4;
+    if (width > 0 && width <= INT_MAX / 4) {
+      rowSize = width * 4;
+    } else {
+      rowSize = -1;
+    }
     break;
 #if SPLASH_CMYK
   case splashModeCMYK8:
-    rowSize = width * 4;
+    if (width > 0 && width <= INT_MAX / 4) {
+      rowSize = width * 4;
+    } else {
+      rowSize = -1;
+    }
     break;
 #endif
   }
-  rowSize += rowPad - 1;
-  rowSize -= rowSize % rowPad;
+  if (rowSize > 0) {
+    rowSize += rowPad - 1;
+    rowSize -= rowSize % rowPad;
+  }
   data = (SplashColorPtr)gmallocn(rowSize, height);
   if (!topDown) {
     data += (height - 1) * rowSize;
@@ -244,7 +272,7 @@ Guchar SplashBitmap::getAlpha(int x, int y) {
   return alpha[y * width + x];
 }
 
-SplashError SplashBitmap::writePNGFile(char *fileName) {
+SplashError SplashBitmap::writeImgFile(SplashImageFileFormat format, char *fileName, int hDPI, int vDPI) {
   FILE *f;
   SplashError e;
 
@@ -252,25 +280,48 @@ SplashError SplashBitmap::writePNGFile(char *fileName) {
     return splashErrOpenFile;
   }
 
-  e = writePNGFile(f);
+  e = writeImgFile(format, f, hDPI, vDPI);
   
   fclose(f);
   return e;
 }
 
-SplashError SplashBitmap::writePNGFile(FILE *f) {
-#ifndef ENABLE_LIBPNG
-  error(-1, "PNG support not compiled in");
-  return splashErrGeneric;
-#else
-  if (mode != splashModeRGB8 && mode != splashModeMono8 && mode != splashModeMono1) {
+SplashError SplashBitmap::writeImgFile(SplashImageFileFormat format, FILE *f, int hDPI, int vDPI) {
+  ImgWriter *writer;
+	SplashError e;
+  
+  switch (format) {
+    #ifdef ENABLE_LIBPNG
+    case splashFormatPng:
+	  writer = new PNGWriter();
+      break;
+    #endif
+
+    #ifdef ENABLE_LIBJPEG
+    case splashFormatJpeg:
+      writer = new JpegWriter();
+      break;
+    #endif
+	
+    default:
+      // Not the greatest error message, but users of this function should
+      // have already checked whether their desired format is compiled in.
+      error(-1, "Support for this image type not compiled in");
+      return splashErrGeneric;
+  }
+
+	e = writeImgFile(writer, f, hDPI, vDPI);
+	delete writer;
+	return e;
+}
+
+SplashError SplashBitmap::writeImgFile(ImgWriter *writer, FILE *f, int hDPI, int vDPI) {
+  if (mode != splashModeRGB8 && mode != splashModeMono8 && mode != splashModeMono1 && mode != splashModeXBGR8) {
     error(-1, "unsupported SplashBitmap mode");
     return splashErrGeneric;
   }
 
-  PNGWriter *writer = new PNGWriter();
-  if (!writer->init(f, width, height)) {
-    delete writer;
+  if (!writer->init(f, width, height, hDPI, vDPI)) {
     return splashErrGeneric;
   }
 
@@ -278,25 +329,44 @@ SplashError SplashBitmap::writePNGFile(FILE *f) {
     case splashModeRGB8:
     {
       SplashColorPtr row;
-      png_bytep *row_pointers = new png_bytep[height];
+      unsigned char **row_pointers = new unsigned char*[height];
       row = data;
 
       for (int y = 0; y < height; ++y) {
         row_pointers[y] = row;
         row += rowSize;
       }
-      if (!writer->writePointers(row_pointers)) {
+      if (!writer->writePointers(row_pointers, height)) {
         delete[] row_pointers;
-        delete writer;
         return splashErrGeneric;
       }
       delete[] row_pointers;
     }
     break;
     
+    case splashModeXBGR8:
+    {
+      unsigned char *row = new unsigned char[3 * width];
+      for (int y = 0; y < height; y++) {
+        // Convert into a PNG row
+        for (int x = 0; x < width; x++) {
+          row[3*x] = data[y * rowSize + x * 4 + 2];
+          row[3*x+1] = data[y * rowSize + x * 4 + 1];
+          row[3*x+2] = data[y * rowSize + x * 4];
+        }
+
+        if (!writer->writeRow(&row)) {
+          delete[] row;
+          return splashErrGeneric;
+        }
+      }
+      delete[] row;
+    }
+    break;
+    
     case splashModeMono8:
     {
-      png_byte *row = new png_byte[3 * width];
+      unsigned char *row = new unsigned char[3 * width];
       for (int y = 0; y < height; y++) {
         // Convert into a PNG row
         for (int x = 0; x < width; x++) {
@@ -307,7 +377,6 @@ SplashError SplashBitmap::writePNGFile(FILE *f) {
 
         if (!writer->writeRow(&row)) {
           delete[] row;
-          delete writer;
           return splashErrGeneric;
         }
       }
@@ -317,7 +386,7 @@ SplashError SplashBitmap::writePNGFile(FILE *f) {
     
     case splashModeMono1:
     {
-      png_byte *row = new png_byte[3 * width];
+      unsigned char *row = new unsigned char[3 * width];
       for (int y = 0; y < height; y++) {
         // Convert into a PNG row
         for (int x = 0; x < width; x++) {
@@ -328,7 +397,6 @@ SplashError SplashBitmap::writePNGFile(FILE *f) {
 
         if (!writer->writeRow(&row)) {
           delete[] row;
-          delete writer;
           return splashErrGeneric;
         }
       }
@@ -342,12 +410,8 @@ SplashError SplashBitmap::writePNGFile(FILE *f) {
   }
   
   if (writer->close()) {
-    delete writer;
     return splashErrGeneric;
   }
 
-  delete writer;
-
   return splashOk;
-#endif
 }
