@@ -13,7 +13,8 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2007-2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -41,6 +42,7 @@
 #include "Catalog.h"
 #include "Page.h"
 #include "PDFDoc.h"
+#include "PDFDocFactory.h"
 #include "HtmlOutputDev.h"
 #include "PSOutputDev.h"
 #include "GlobalParams.h"
@@ -187,7 +189,13 @@ int main(int argc, char *argv[]) {
 
   fileName = new GooString(argv[1]);
 
-  doc = new PDFDoc(fileName, ownerPW, userPW);
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
+  }
+
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
   if (userPW) {
     delete userPW;
   }
@@ -210,19 +218,28 @@ int main(int argc, char *argv[]) {
   // construct text file name
   if (argc == 3) {
     GooString* tmp = new GooString(argv[2]);
-    p=tmp->getCString()+tmp->getLength()-5;
-    if (!xml)
-      if (!strcmp(p, ".html") || !strcmp(p, ".HTML"))
-	htmlFileName = new GooString(tmp->getCString(),
-				   tmp->getLength() - 5);
-      else htmlFileName =new GooString(tmp);
-    else   
-      if (!strcmp(p, ".xml") || !strcmp(p, ".XML"))
-	htmlFileName = new GooString(tmp->getCString(),
-				   tmp->getLength() - 5);
-      else htmlFileName =new GooString(tmp);
-    
+    if (!xml) {
+      if (tmp->getLength() >= 5) {
+        p = tmp->getCString() + tmp->getLength() - 5;
+        if (!strcmp(p, ".html") || !strcmp(p, ".HTML")) {
+          htmlFileName = new GooString(tmp->getCString(), tmp->getLength() - 5);
+        }
+      }
+    } else {
+      if (tmp->getLength() >= 4) {
+        p = tmp->getCString() + tmp->getLength() - 4;
+        if (!strcmp(p, ".xml") || !strcmp(p, ".XML")) {
+          htmlFileName = new GooString(tmp->getCString(), tmp->getLength() - 4);
+        }
+      }
+    }
+    if (!htmlFileName) {
+      htmlFileName =new GooString(tmp);
+    }
     delete tmp;
+  } else if (fileName->cmp("fd://0") == 0) {
+      error(-1, "You have to provide an output filename when reading form stdin.");
+      goto error;
   } else {
     p = fileName->getCString() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF"))
@@ -381,6 +398,7 @@ int main(int argc, char *argv[]) {
   // clean up
  error:
   if(doc) delete doc;
+  delete fileName;
   if(globalParams) delete globalParams;
 
   if(htmlFileName) delete htmlFileName;
